SystemOrganization addCategory: #ZipperControl!
SystemOrganization addCategory: #'ZipperControl-Tests'!

!BlockClosure methodsFor: '*ZipperControl' stamp: 'fbs 6/22/2011 07:36'!
compose: aUnaryBlock
	^ [:x | self value: (aUnaryBlock value: x)].! !

Object subclass: #EmptyTestTree
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ZipperControl-Tests'!

!EmptyTestTree methodsFor: 'as yet unclassified' stamp: 'fbs 6/23/2011 16:34'!
collect: aBlock
	^ self.! !

Object subclass: #TestTree
	instanceVariableNames: 'left right value'
	classVariableNames: 'EmptyTree'
	poolDictionaries: ''
	category: 'ZipperControl-Tests'!

!TestTree class methodsFor: 'instance creation' stamp: 'fbs 6/23/2011 16:32'!
empty
	^ EmptyTree! !

!TestTree class methodsFor: 'class initialization' stamp: 'fbs 6/23/2011 16:33'!
initialize
	EmptyTree := EmptyTestTree new.! !

!TestTree class methodsFor: 'instance creation' stamp: 'fbs 6/23/2011 15:16'!
left: aTree right: anotherTree value: anObject
	^ self new left: aTree right: anotherTree value: anObject.! !

!TestTree class methodsFor: 'instance creation' stamp: 'fbs 6/23/2011 16:36'!
value: anObject
	^ self
		left: self empty
		right: self empty
		value: anObject.! !

!TestTree methodsFor: 'enumerating' stamp: 'fbs 7/1/2011 08:33'!
collect: aBlock
	| newValue newLeft newRight |
	newValue := aBlock value: value.
	newLeft := aBlock value: left.
	newRight := aBlock value: right.
	((newValue == value) and: [newLeft == left] and: [newRight == right])
		ifTrue: [^ self].

	^ TestTree
		left: (left collect: aBlock)
		right: (right collect: aBlock)
		value: (aBlock value: value)! !

!TestTree methodsFor: 'accessing' stamp: 'fbs 6/23/2011 16:37'!
left
	^ left.! !

!TestTree methodsFor: 'private' stamp: 'fbs 6/23/2011 16:39'!
left: aTree right: anotherTree value: anObject
	left := aTree.
	right := anotherTree.
	value := anObject.! !

!TestTree methodsFor: 'accessing' stamp: 'fbs 6/23/2011 16:37'!
right
	^ right.! !

!TestTree methodsFor: 'accessing' stamp: 'fbs 6/23/2011 16:37'!
value
	^ value.! !

Object subclass: #Zipper
	instanceVariableNames: 'term continuation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ZipperControl'!

!Zipper commentStamp: 'fbs 6/13/2011 07:47' prior: 0!
I am a suspended walk over an enumeration. That is, given a traversal strategy and a data structure, I allow for the traversal of that data structure. I also allow for copy-on-write mutation that shares structure between the old and new versions of a structure as much as my underlying enumerator allows.

In particular, my enumerator either returns false or a replacement node.!

Zipper subclass: #ZipDone
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ZipperControl'!

!ZipDone class methodsFor: 'instance creation' stamp: 'fbs 6/22/2011 07:48'!
on: anObject
	^ self new term: anObject.! !

!ZipDone methodsFor: 'testing' stamp: 'fbs 6/22/2011 07:32'!
atEnd
	^ true.! !

!ZipDone methodsFor: 'applying' stamp: 'fbs 6/23/2011 18:02'!
next: anObject
	^ Error signal: 'Past end of iteration'.! !

!ZipDone methodsFor: 'private' stamp: 'fbs 6/14/2011 18:15'!
term: anObject
	term := anObject.! !

!ZipDone methodsFor: 'applying' stamp: 'fbs 6/14/2011 18:39'!
zipUp
	^ term.! !

!Zipper class methodsFor: 'instance creation' stamp: 'fbs 6/23/2011 17:36'!
on: anObject with: aUnaryBlock
	^ self new on: anObject with: aUnaryBlock! !

!Zipper class methodsFor: 'enumerating' stamp: 'fbs 6/28/2011 17:49'!
zipOver: aCollection
	"Turn an enumeration inside out: return a zipper on a traversal such that the continuation associated with the traversal takes a Maybe. If the value's Nothing, leave the value associated with the zipper unchanged; otherwise, replace it with the Just value."
	| f zippers |
	"Return a new Zipper for each element in aCollection. As a termination marker, return a final zipper, a ZipDone, containing the new collection of resultant zippers."
	f := [:a | [:k | Zipper on: a with: (k compose: [:x | x maybe: #yourself default: a])] shift].
	^ [ZipDone on: (aCollection collect: [:each | f value: each])] reset.! !

!Zipper methodsFor: 'testing' stamp: 'fbs 6/22/2011 07:32'!
atEnd
	^ false.! !

!Zipper methodsFor: 'accessing' stamp: 'fbs 6/17/2011 07:46'!
continuation
	^ continuation.! !

!Zipper methodsFor: 'applying' stamp: 'fbs 6/28/2011 07:38'!
next: aMaybe
	^ [continuation value: aMaybe] reset.! !

!Zipper methodsFor: 'private' stamp: 'fbs 6/23/2011 17:33'!
on: anObject with: aBinaryBlock
	term := anObject.
	continuation := aBinaryBlock.! !

!Zipper methodsFor: 'accessing' stamp: 'fbs 6/23/2011 17:32'!
term
	^ term! !

!Zipper methodsFor: 'applying' stamp: 'fbs 6/28/2011 11:13'!
zipUp
	"Zip over the remaining terms in the traversal, leaving them unchanged."
	^ [continuation value: Maybe nothing] reset.! !

TestCase subclass: #TestTreeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ZipperControl-Tests'!

!TestTreeTest methodsFor: 'testing - enumerating' stamp: 'fbs 6/23/2011 16:32'!
testCollectOnEmptyTreeReturnsEmptyTree
	self assert: (TestTree empty collect: #printString) = TestTree empty.! !

!TestTreeTest methodsFor: 'testing - enumerating' stamp: 'fbs 6/23/2011 16:37'!
testCollectOnTreeReturnsTree
	| in out |
	in := TestTree left: (TestTree value: 2) right: (TestTree value: 3) value: 1.
	out := in collect: #printString.
	self assert: out value = '1'.
	self assert: out left value = '2'.
	self assert: out right value = '3'.! !

!TestTreeTest methodsFor: 'testing - enumerating' stamp: 'fbs 7/1/2011 08:31'!
testCollectingSharesUnaffectedNodes
	| t1 t2 |
	t1 := TestTree left: (TestTree value: 1) right: (TestTree value: 2) value: 0.
	t2 := t1 collect: #yourself.
	self assert: t1 left == t2 left.
	self assert: t1 right == t2 right.
	self assert: t1 == t2.! !

TestCase subclass: #ZipperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ZipperControl-Tests'!

!ZipperTest methodsFor: 'testing - enumerating' stamp: 'fbs 7/1/2011 08:40'!
testEnumerationOnSharingTraversalUsingNothingReturnsIdenticalStructure
	| t t2 z |
	t := TestTree left: (TestTree value: 2) right: (TestTree value: 1) value: 0.
	z := Zipper zipOver: t.
	[z atEnd] whileFalse: [z := z next: Maybe nothing].

	t2 := z zipUp.
	self assert: t2 left == t left.
	self assert: t2 right == t right.
	self assert: t2 == t.
	! !

!ZipperTest methodsFor: 'testing - enumerating' stamp: 'fbs 6/23/2011 22:13'!
testEnumerationUsingJustReplacesValues
	| coll z |
	coll := #(1 2 3 4 5).
	z := Zipper zipOver: coll.
	[z atEnd] whileFalse: [z := z next: (Just value: 1)].
	self assert: z zipUp = #(1 1 1 1 1).! !

!ZipperTest methodsFor: 'testing - enumerating' stamp: 'fbs 6/23/2011 22:21'!
testEnumerationUsingNothingSimplyEnumerates
	| coll z i |
	coll := #(1 2 3 4 5).
	z := Zipper zipOver: coll.
	i := 0.
	[z atEnd] whileFalse: [z := z next: Maybe nothing. i := i + 1].
	self assert: z zipUp = coll.
	self assert: i = 5.! !

!ZipperTest methodsFor: 'testing - enumerating' stamp: 'fbs 6/23/2011 17:53'!
testEnumerationWorksOverTrees
	| coll z out |
	coll := TestTree left: (TestTree left: (TestTree value: 3) right: (TestTree value: 4) value: 2) right: (TestTree left: (TestTree value: 6) right: (TestTree value: 7) value: 5) value: 1.
	z := Zipper zipOver: coll.
	[z atEnd] whileFalse: [z := z next: Maybe nothing].
	out := z zipUp.
	self assert: out value = coll value.
	self assert: out left value = coll left value.
	self assert: out right value = coll right value.
	self assert: out left left value = coll left left value.
	self assert: out left right value = coll left right value.
	self assert: out right left value = coll right left value.
	self assert: out right right value = coll right right value.! !

!ZipperTest methodsFor: 'testing - enumerating' stamp: 'fbs 6/23/2011 22:10'!
testZipOverReturnsSuspendedWalk
	| coll z1 z2 z3 z4 z5 z6 |
	coll := #(1 2 3 4 5).
	z1 := Zipper zipOver: coll.
	z2 := z1 next: Maybe nothing.
	z3 := z2 next: Maybe nothing.
	z4 := z3 next: Maybe nothing.
	z5 := z4 next: Maybe nothing.
	z6 := z5 next: Maybe nothing.
	self assert: (z1 term = 1).
	self assert: (z2 term = 2).
	self assert: (z3 term = 3).
	self assert: (z4 term = 4).
	self assert: (z5 term = 5).
	self assert: z6 atEnd.! !
TestTree initialize!
